# -*- coding:utf-8 -*-
# CREATED BY: bohuai jiang 
# CREATED ON: 2020/8/12 4:42 PM
# LAST MODIFIED ON:
# AIM:
import re

SYMBOLS = {
    'quotation_left': re.compile('([“])'),
    'quotation_en': re.compile('(["])'),
    'quotation_right': re.compile('([”])'),
    'bracket_left': re.compile('([<{\[\(（【「])'),
    'bracket_right': re.compile('([\)\]}」】）>])'),
    'book_left': re.compile('([《])'),
    'book_right': re.compile('([》])'),
    'comma': re.compile('([,，;])'),
    'end_symbols': re.compile('([?\!…;？！。])'),
    'en_dot': re.compile('\.'),
    'all_symbols': re.compile('([\]\(【\?】。,！[”，\!\.「<>？"（“…《）}》」;:：\){\)\'\s])')
}


SYMBOLS_EN = {
    'quotation_left': re.compile('([“])'),
    'quotation_en': re.compile('(["])'),
    'quotation_right': re.compile('([”])'),
    'all_quota': re.compile('([“"”])'),
    'bracket_left': re.compile('([<{\[\(（【「])'),
    'bracket_right': re.compile('([\)\]}」】）>])'),
    'book_left': re.compile('([《])'),
    'book_right': re.compile('([》])'),
    'comma': re.compile('([,，;])'),
    'end_symbols': re.compile('([?\!…;？！。])'),
    'en_dot': re.compile('\.'),
    'all_symbols': re.compile('([\]\(【\?】。,！[”，\!\.「<>？"（“…《）}》」;:：\){\)\'\s])')
}



# -*- coding:utf-8 -*-
# CREATED BY: bohuai jiang
# CREATED ON: 2020/8/4 2:52 PM
# LAST MODIFIED ON:
# AIM: automata abstract class
from abc import ABC, abstractmethod

from .sequence import StrSequence


class Criteria(ABC):
    def __init__(self, name: str, symbols: dict, reverse: bool = False, ):
        self.name = name
        self.revers = reverse
        self.symbols = symbols

    @abstractmethod
    def accept(self, state: StrSequence) -> bool:
        assert (0)

    def __call__(self, state: StrSequence):
        if self.revers:
            return not self.accept(state)
        else:
            return self.accept(state)


class Operation(ABC):
    def __init__(self, name: str, symbols: dict):
        self.name = name
        self.symbols = symbols

    @abstractmethod
    def operate(self, state: StrSequence) -> StrSequence:
        assert (0)

    def __call__(self, state: StrSequence) -> StrSequence:
        return self.operate(state)



# -*- coding:utf-8 -*-
# CREATED BY: jiangbohuai
# CREATED ON: 2021/1/21 3:53 PM
# LAST MODIFIED ON:
# AIM:

from sentence_spliter.spliter import cut_to_sentences_en

with open('file.txt' , 'r') as f:
    sentence = f.read().strip()

out = cut_to_sentences_en(sentence)

print('\n\n'.join(v.strip() for v in out))



# -*- coding:utf-8 -*-
# CREATED BY: jiangbohuai
# CREATED ON: 2021/1/21 5:23 PM
# LAST MODIFIED ON:
# AIM:
import attrdict

from .automata import condition, operation
from .automata import symbols

SYMBOLS = symbols.SYMBOLS_EN


def init_nodes(**kwargs):
    max_len = kwargs.get('max_len', 25)
    min_len = kwargs.get('min_len', 5)
    # hard_max = kwargs.get('hard_max', 200)

    # --- initialize condition & operation --- #
    edges = attrdict.AttrDict({
        'is_end_symbol': condition.IsEndSymbolEN(SYMBOLS),
        'is_bracket_close': condition.IsBracketClose(SYMBOLS),
        'is_book_close': condition.IsBookClose(SYMBOLS),
        'is_quote_close': condition.IsQuoteClose(SYMBOLS),
        'is_end_state': condition.IsEndState(SYMBOLS),
        'is_long_sentence': condition.IsLongSentence(SYMBOLS, max_len=max_len),
        'is_not_short_sentence': condition.IsShortSentence(SYMBOLS, min_len=min_len, reverse=True),
        'is_next_with_space': condition.IsNextWithSpace(SYMBOLS),

        # --- sepcial Condition --- #
        'special_ends': condition.SpecialEnds(),
        'not_in_white_list': condition.NotInWhitelistDotEn(),
        'is_empty': condition.IsSpace(SYMBOLS),
        'is_left_quota': condition.IsLeftQuotaEn(SYMBOLS),
        'is_right_quota': condition.IsRightQuotaEn(SYMBOLS),
        'dialogue_is_greater_than': condition.DialogueIsGreaterThanEN(SYMBOLS, length=min_len)
    })

    nodes = attrdict.AttrDict({
        'do_cut': operation.Normal(SYMBOLS, name='do_cut'),
        'cut_pre_idx': operation.CutPreIdx(SYMBOLS, name='cut_per_idx'),
        'long_handler': operation.LongHandlerEN(SYMBOLS, min_sentence_length=min_len, name='long_handler'),
        'init': operation.Indolent(SYMBOLS),
        'quota_handler': operation.Indolent(SYMBOLS, name='quota_handler'),
        'end': operation.EndState(SYMBOLS)
    })
    return edges, nodes


def long_cuter_en(**kwargs):
    edges, nodes = init_nodes(**kwargs)
    return {
        nodes.init: [
            {'edge': edges.is_end_state,
             'node': nodes.end},

            # -- "..." "..."  -- #
            {'edge': [edges.is_right_quota,
                      edges.dialogue_is_greater_than],
             'args': all,
             'node': nodes.quota_handler},

            # {'edge': edges.special_ends,
            #  'node': nodes.do_cut},

            # -- normal cut -- #
            {'edge': [
                edges.is_not_short_sentence,
                edges.is_next_with_space,
                edges.is_end_symbol,
                edges.is_bracket_close,
                edges.is_quote_close,
                edges.not_in_white_list
            ],
                'args': all,
                'node': nodes.do_cut},

            {'edge': edges.is_long_sentence,
             'node': nodes.long_handler},

        ],

        nodes.quota_handler: [
            {'edge': edges.is_end_state,
             'node': nodes.end},

            {'edge': [
                edges.is_not_short_sentence,
                edges.is_left_quota,
            ],
                'args': all,
                'node': nodes.cut_pre_idx},

            {'edge': edges.is_empty,
             'node': nodes.quota_handler},

            {'edge': [
                edges.is_not_short_sentence,
                edges.is_end_symbol,
                edges.is_bracket_close,
                edges.is_book_close,
                edges.is_quote_close],
                'args': all,
                'node': nodes.cut_pre_idx},  # do_cut},

            {'edge': None,
             'node': nodes.init}
        ],

        nodes.cut_pre_idx: [
            {'edge': edges.is_end_state,
             'node': nodes.end},

            {'edge': None,  # else 状态
             'node': nodes.init},
        ],

        nodes.do_cut: [
            {'edge': edges.is_end_state,
             'node': nodes.end},

            {'edge': None,  # else 状态
             'node': nodes.init},
        ],
        nodes.long_handler: [
            {'edge': edges.is_end_state,
             'node': nodes.end},

            {'edge': None,  # else 状态
             'node': nodes.init},
        ]
    }



# -*- coding:utf-8 -*-
# CREATED BY: jiangbohuai
# CREATED ON: 2021/4/8 11:04 AM
# LAST MODIFIED ON:
# AIM:
import unittest
from sentence_spliter.spliter import cut_to_sentences_en
from sentence_spliter.logic_graph_en import long_cuter_en
from sentence_spliter.automata.sequence import EnSequence
from sentence_spliter.automata.state_machine import StateMachine


class test_spliter(unittest.TestCase):
    def test_bug_2021_4_10(self):
        paragraph = "Some of the fellows pretended to think I was mad when I rushed at Chandos and hugged him, and shouted, \"It's all your doing, old fellow. I'm going to sea! I'm going to sea!\""
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=10))
        m_input = EnSequence(paragraph)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = ['The night was rapidly approaching; and already, at the cry of “Moccoletti!” '
                  'repeated by the shrill voices of a thousand vendors,',
                  ' two or three stars began to burn among the crowd.',
                  ' It was a signal. At the end of ten minutes fifty thousand lights glittered, '
                  'descending from the Palazzo di Venezia to the Piazza del Popolo,',
                  ' and mounting from the Piazza del Popolo to the Palazzo di Venezia.',
                  ' It seemed like the fête of Jack-o’-lanterns.']

        self.assertEqual(expect, acutal)


# from sentence_spliter.sentence_spliter import Spliter
from sentence_spliter.sentence_spliter import Spliter
from loguru import logger


def test_spliter():
    options = {
        'language': 'zh',
        'long_short_sent_handle': 'y',
        'hard_max_length': 300,
        'max_length': 150,
        'min_length': 15,
        'remove_blank': 'True'
    }
    spliter = Spliter(options)
    input_path = '../test/data/test.txt'
    output_path = '../test/data/cut_sentences.txt'
    with open(input_path, 'r', encoding='utf-8') as f_read:
        sentence = f_read.read()
    logger.debug(sentence)
    output_lines = []
    output_lines.extend(spliter.cut_to_sentences(sentence))
    open(output_path, 'w', encoding='utf-8').write('\n'.join(output_lines))


if __name__ == '__main__':
    test_spliter()
